<?php if (!defined('PmWiki')) exit();
/*  Copyright 2004, 2007 Patrick R. Michaud (pmichaud@pobox.com)
	Copyright 2011 Tamara Temple (tamara@tamaratemple.com)

	CHANGED: modified to allow persistance of skincolor and skintheme for Choice theme (tpt) 2011-06-12

    The easy way to use this script is to do following in 
    local/config.php: 

        $EnableAutoSkinColorList = 1;
        include_once('cookbook/colorchange.php');

    This will allow browser to choose any skin color that exists
    in the pub/skins/SKINNAME/color or $FarmD/pub/skins/SKINNAME/color directory.
	(Note: this directory scheme is used by a few pmwiki skins:
	Choice, Colorimetry, Skittlish). Valid skins can be set
	by creating an array, $ColorChangerValidColors:
	
		$ValidColorChangerColors = array(
			'choice',
			'colorimetry',
			'skittlish',
		);
		
    To only allow a specific set of colors and themes, create an array
    called $PageColorList and $PageStyleList that maps names from ?color= or
    ?style= into the corresponding skin to be used for that
    name.  For example:
    
		$PageColorList = array(
			'blue_bold' => 'blue_bold',
			'blue_muted' => 'blue_muted',
			'green_bold' => 'green_bold',
		);
		$PageStyleList = array(
			'light' => 'light',
			'dark' => 'light',
		);

    If $EnableAutoColorList is also set, then entries in
    $PageColorList take precedence.  If a browser requests
    a skin that isn't available, then PmWiki defaults to
    the skin already defined by $Skin.

	The same goes for $EnableAutoStyleList.
	
    By default, the setcolor and setstyle cookies that are created will expire after
    one year.  You can set it to expire at the end of the browser
    session by setting $ColorCookieExpires=0 and/or $StyleCookieExpires=0;

	These variables, if used, must be set before including the recipe.
	
	TODO: Deal with conflicting query string parameters color:setcolor and skintheme:setskintheme
	
*/
# @sms(__FILE__.'@'.__LINE__." enter.");

SDV($RecipeInfo['ColorChange']['Version'], '2011-06-12');
SDV($ColorCookie, $CookiePrefix.'color');
SDV($ColorCookieExpires, $Now+60*60*24*365);
SDV($StyleCookie, $CookiePrefix.'style');
SDV($StyleCookieExpires, $Now+60*60*24*365);
SDV($ValidColorChangerSkins, array('choice','colorimetry','skittlish'));
SDV($PageColorList, cc_BuildColorList($Skin));
SDV($PageStyleList, array('light'=>'light','dark'=>'dark'));

# @sms(__FILE__.'@'.__LINE__." ValidColorChangerSkins=");
# @sms($ValidColorChangerSkins);

# Only process changes for valid skins
$ColorChanger_Proceed = FALSE;  # the default case
if (isset($ValidColorChangerSkins)) {
	if (in_array($Skin,$ValidColorChangerSkins)) {$ColorChanger_Proceed = TRUE;}
}

if ($ColorChanger_Proceed) {
# 	@sms(__FILE__.'@'.__LINE__." Proceeding to process query strings.");
	# Process any color changes
	# Get existing color from cookie, if any
	if (isset($_COOKIE[$ColorCookie])) $co = $_COOKIE[$ColorCookie];
	# Check if we're setting color changes permanently
	if (isset($_GET['setcolor'])) {
		$co = $_GET['setcolor'];
		setcookie($ColorCookie, $co, $ColorCookieExpires, '/');
# 		@sms(__FILE__.'@'.__LINE__." just set color cookie");
	}
	# Set temporary color scheme
	if (isset($_GET['color'])) $co = $_GET['color'];
# 	@sms(__FILE__.'@'.__LINE__." co=$co");

	##  If $EnableAutoColorList is set, then we accept any color that
	##  exists in pub/skins/SKIN/color or $FarmD/pub/skins/SKIN/color.
	if (IsEnabled($EnableAutoSkinColorList, 0) 
		&& preg_match('/^[_\\w]+$/', $co)
		&& (file_exists("pub/skins/$Skin/color/$co".'.css') || file_exists("$FarmD/pub/skins/$Skin/color/$co".'.css')))
		{
			$SkinColor = $co;
# 			@sms(__FILE__.'@'.__LINE__." should have just reset SkinColor=$SkinColor");
		} else {
# 			@sms(__FILE__.'@'.__LINE__." color was not set.");
			
		}
	

	##  If there's a specific mapping in $PageColorList, we use it no
	##  matter what.
	if (@$PageColorList[$co]) $SkinColor = $PageColorList[$co];
# 	@sms(__FILE__.'@'.__LINE__." may have just reset SkinColor=$SkinColor");

	# Proces style changes
	# Get existing style from cookie, if any
	if (isset($_COOKIE[$StyleCookie])) $st = $_COOKIE[$StyleCookie];
	# Check if we're setting style changes permanently
	if (isset($_GET['setskintheme'])) {
		$st = $_GET['setskintheme'];
		setcookie($StyleCookie, $st, $StyleCookieExpires, '/');
# 		@sms(__FILE__.'@'.__LINE__." set StyleCookie");
	}
	# Set temporary style sheme
	if (isset($_GET['skintheme'])) $st = $_GET['skintheme'];
# 	@sms(__FILE__.'@'.__LINE__." May have set st=$st");
	
	## Verify that $EnableAutoStyleList is set. (At this point, the themes
    ## in this recipes effect have no real way to determine which are
    ## valid skin styles)
	if (IsEnabled($EnableAutoSkinColorList, 0) 
		&& preg_match('/^[\\w]+$/', $co))
		$SkinTheme = $st;
# 	@sms(__FILE__.'@'.__LINE__." May have set SkinTheme=$SkinTheme");

	##  If there's a specific mapping in $PageStyleList, we use it no
	##  matter what.
	if (@$PageStyleList[$st]) $SkinTheme = $PageStyleList[$st];
# 	@sms(__FILE__.'@'.__LINE__." May have just reset SkinTheme=$SkinTheme");

}

# @sms(__FILE__.'@'.__LINE__." Ending colorchange. Skin=$Skin. SkinColor=$SkinColor. SkinTheme=$SkinTheme.");

// =====================
// = Utility Functions =
// =====================

/**
 * Build the set of valid colors under the current skin, return hash of 'color'=>'color'
 *
 * @return hash of 'color'='color' containing each installed color scheme for this skin
 * @author Tamara Temple tamara@tamaratemple.com
 **/
function cc_BuildColorList($sk)
{
# 	@sms(__FILE__.'@'.__LINE__." Entering cc_BuildColorList. sk=$sk");
	if (isset($sk) && (is_dir("pub/skins/$sk") || is_dir("$FarmD/pub/skins/$sk"))) {
		if (is_dir("pub/skins/$sk/color")) {
			$co_dir = "pub/skins/$sk/color";
		} else if (is_dir("$FarmD/pub/skins/$sk/color")) {
			$co_dir = "$FarmD/pub/skins/$sk/color";
		}
# 		@sms(__FILE__.'@'.__LINE__." In cc_BuildColorList. co_dir=$co_dir");
		if (isset($co_dir)) {
			$co_dir_file_list=array();
			$co_dir_files = scandir($co_dir);
			foreach ($co_dir_files as $co_dir_file) {
				if (preg_match('/\.css$/',$co_dir_file)) {
					$colorname = str_replace('.css','',$co_dir_file);
					$colorlist[$colorname]= $colorname;
				}
			}
# 			@sms(__FILE__.'@'.__LINE__." In cc_BuidColorList. colorlist:");
# 			@sms($colorlist);
		}
	}
	return (isset($co_dir_file_list) && !empty($co_dir_file_list)) ? $co_dir_file_list : 0;
}
